@@ -15,6 +15,13 @@ class MCQuestion(Question):
     def get_answers(self):
         return Answer.objects.filter(question=self)
 
+    def get_answers_list(self):
+        return [(answer.id, answer.content) for answer in
+                Answer.objects.filter(question=self)]
+
+    def answer_choice_to_string(self, guess):
+        return Answer.objects.get(id=guess).content
+
     class Meta:
         verbose_name = "Multiple Choice Question"
         verbose_name_plural = "Multiple Choice Questions"
