@@ -5,25 +5,21 @@ from django.contrib.admin.widgets import FilteredSelectMultiple
 from .models import Quiz, Category, Progress, Question
 from multichoice.models import MCQuestion, Answer
 from true_false.models import TF_Question
-
-
-class QuestionInline(admin.TabularInline):
-    model = Question.quiz.through
-    filter_horizontal = ('content',)
+from essay.models import Essay_Question
 
 
 class AnswerInline(admin.TabularInline):
     model = Answer
 
-"""
-below is from
-http://stackoverflow.com/questions/11657682/
-django-admin-interface-using-horizontal-filter-with-
-inline-manytomany-field
-"""
-
 
 class QuizAdminForm(forms.ModelForm):
+    """
+    below is from
+    http://stackoverflow.com/questions/11657682/
+    django-admin-interface-using-horizontal-filter-with-
+    inline-manytomany-field
+    """
+
     class Meta:
         model = Quiz
 
@@ -31,21 +27,20 @@ class QuizAdminForm(forms.ModelForm):
         queryset=Question.objects.all().select_subclasses(),
         required=False,
         widget=FilteredSelectMultiple(
-            verbose_name=('Questions'),
+            verbose_name='Questions',
             is_stacked=False))
 
     def __init__(self, *args, **kwargs):
         super(QuizAdminForm, self).__init__(*args, **kwargs)
         if self.instance.pk:
-            self.fields['questions'].initial = self.instance.question_set.all()
+            self.fields['questions'].initial =\
+                self.instance.question_set.all().select_subclasses()
 
     def save(self, commit=True):
         quiz = super(QuizAdminForm, self).save(commit=False)
-        if commit:
-            quiz.save()
-        if quiz.pk:
-            quiz.question_set = self.cleaned_data['questions']
-            self.save_m2m()
+        quiz.save()
+        quiz.question_set = self.cleaned_data['questions']
+        self.save_m2m()
         return quiz
 
 
@@ -66,7 +61,7 @@ class MCQuestionAdmin(admin.ModelAdmin):
     list_filter = ('category',)
     fields = ('content', 'category', 'quiz', 'explanation')
 
-    search_fields = ('content', )
+    search_fields = ('content', 'explanation')
     filter_horizontal = ('quiz',)
 
     inlines = [AnswerInline]
@@ -88,8 +83,17 @@ class TFQuestionAdmin(admin.ModelAdmin):
     search_fields = ('content', 'explanation')
     filter_horizontal = ('quiz',)
 
+
+class EssayQuestionAdmin(admin.ModelAdmin):
+    list_display = ('content', 'category', )
+    list_filter = ('category',)
+    fields = ('content', 'category', 'quiz', 'explanation', )
+    search_fields = ('content', 'explanation')
+    filter_horizontal = ('quiz',)
+
 admin.site.register(Quiz, QuizAdmin)
 admin.site.register(Category, CategoryAdmin)
 admin.site.register(MCQuestion, MCQuestionAdmin)
 admin.site.register(Progress, ProgressAdmin)
 admin.site.register(TF_Question, TFQuestionAdmin)
+admin.site.register(Essay_Question, EssayQuestionAdmin)
