@@ -1,5 +1,10 @@
 import re
+import json
+
 from django.db import models
+from django.core.exceptions import ValidationError
+from django.core.validators import MaxValueValidator
+
 from model_utils.managers import InheritanceManager
 
 
@@ -64,7 +69,8 @@ class Quiz(models.Model):
                                      default=False,
                                      help_text="If yes, the result of each"
                                                " attempt by a user will be"
-                                               " stored.")
+                                               " stored. Necessary for"
+                                               " marking.")
 
     single_attempt = models.BooleanField(blank=False,
                                          default=False,
@@ -73,6 +79,19 @@ class Quiz(models.Model):
                                                    " permitted. Non users"
                                                    " cannot sit this exam.")
 
+    pass_mark = models.SmallIntegerField(blank=True,
+                                         default=0,
+                                         help_text="Percentage required to"
+                                                   " pass exam.",
+                                         validators=[
+                                             MaxValueValidator(100)])
+
+    success_text = models.TextField(blank=True,
+                                    help_text="Displayed if user passes.")
+
+    fail_text = models.TextField(blank=True,
+                                 help_text="Displayed if user fails.")
+
     def save(self, force_insert=False, force_update=False, *args, **kwargs):
         self.url = re.sub('\s+', '-', self.url).lower()
 
@@ -82,6 +101,9 @@ class Quiz(models.Model):
         if self.single_attempt is True:
             self.exam_paper = True
 
+        if self.pass_mark > 100:
+            raise ValidationError(u'%s is above 100' % self.pass_mark)
+
         super(Quiz, self).save(force_insert, force_update, *args, **kwargs)
 
     class Meta:
@@ -104,11 +126,6 @@ class Quiz(models.Model):
     def anon_q_list(self):
         return str(self.id) + "_q_list"
 
-"""
-Progress is used to track an individual signed in users score on different
-quiz's and categories
-"""
-
 
 class ProgressManager(models.Manager):
 
@@ -121,8 +138,8 @@ class ProgressManager(models.Manager):
 
 class Progress(models.Model):
     """
-    Currently stores the score for each category, max possible they could
-    have got, and previous exam paper scores.
+    Progress is used to track an individual signed in users score on different
+    quiz's and categories
 
     Data stored in csv using the format:
         category, score, possible, category, score, possible, ...
@@ -174,10 +191,7 @@ class Progress(models.Model):
                 output[cat.category] = [0, 0]
 
         if len(self.score) > len(score_before):
-            """
-            If a new category has been added, save changes. Otherwise nothing
-            will be saved.
-            """
+            # If a new category has been added, save changes.
             self.save()
 
         return output
@@ -240,9 +254,7 @@ class Progress(models.Model):
             self.save()
 
         else:
-            """
-            if not present but existing category, add with the points passed in
-            """
+            #  if not present but existing, add with the points passed in
             self.score += (str(category) + "," +
                            str(score_to_add) + "," +
                            str(possible_to_add) + ",")
@@ -277,10 +289,27 @@ class SittingManager(models.Manager):
                                   question_list=questions,
                                   incorrect_questions="",
                                   current_score=0,
-                                  complete=False)
+                                  complete=False,
+                                  user_answers='{}')
         new_sitting.save()
         return new_sitting
 
+    def user_sitting(self, user, quiz):
+        if quiz.single_attempt is True and self.filter(user=user,
+                                                       quiz=quiz,
+                                                       complete=True)\
+                                               .count() > 0:
+            return False
+
+        try:
+            sitting = self.get(user=user, quiz=quiz, complete=False)
+        except Sitting.DoesNotExist:
+            sitting = self.new_sitting(user, quiz)
+        except Sitting.MultipleObjectsReturned:
+            sitting = self.filter(user=user, quiz=quiz, complete=False)[0]
+        finally:
+            return sitting
+
 
 class Sitting(models.Model):
     """
@@ -293,6 +322,9 @@ class Sitting(models.Model):
     Incorrect_questions is a list in the same format.
 
     Sitting deleted when quiz finished unless quiz.exam_paper is true.
+
+    User_answers is a json object in which the question PK is stored
+    with the answer the user gave.
     """
 
     user = models.ForeignKey('auth.User')
@@ -308,8 +340,13 @@ class Sitting(models.Model):
 
     complete = models.BooleanField(default=False, blank=False)
 
+    user_answers = models.TextField(blank=True, default='{}')
+
     objects = SittingManager()
 
+    class Meta:
+        permissions = (("view_sittings", "Can see completed exams."),)
+
     def get_first_question(self):
         """
         Returns the next question.
@@ -329,7 +366,7 @@ class Sitting(models.Model):
             self.save()
 
     def add_to_score(self, points):
-        self.current_score = self.get_current_score + int(points)
+        self.current_score += int(points)
         self.save()
 
     @property
@@ -362,16 +399,55 @@ class Sitting(models.Model):
         Adds uid of incorrect question to the list.
         The question object must be passed in.
         """
-        if isinstance(question, Question) is False:
-            return False
+        if len(self.incorrect_questions) > 0:
+            self.incorrect_questions += ','
         self.incorrect_questions += str(question.id) + ","
+        if self.complete:
+            self.add_to_score(-1)
         self.save()
 
+    @property
     def get_incorrect_questions(self):
         """
-        Returns a list of non empty strings
+        Returns a list of non empty integers, representing the pk of
+        questions
         """
-        return filter(None, self.incorrect_questions.split(','))
+        return [int(q) for q in self.incorrect_questions.split(',') if q]
+
+    def remove_incorrect_question(self, question):
+        current = self.get_incorrect_questions
+        current.remove(question.id)
+        self.incorrect_questions = ','.join(map(str, current))
+        self.add_to_score(1)
+        self.save()
+
+    @property
+    def check_if_passed(self):
+        if self.get_percent_correct >= self.quiz.pass_mark:
+            return True
+        else:
+            return False
+
+    @property
+    def result_message(self):
+        if self.check_if_passed:
+            return self.quiz.success_text
+        else:
+            return self.quiz.fail_text
+
+    def add_user_answer(self, question, guess):
+        current = json.loads(self.user_answers)
+        current[question.id] = guess
+        self.user_answers = json.dumps(current)
+        self.save()
+
+    @property
+    def questions_with_user_answers(self):
+        output = {}
+        user_answers = json.loads(self.user_answers)
+        for question in self.quiz.question_set.all().select_subclasses():
+            output[question] = user_answers[unicode(question.id)]
+        return output
 
 
 class Question(models.Model):
