@@ -1,10 +1,11 @@
 from django.conf.urls import patterns, url
 
 from .views import QuizListView, CategoriesListView,\
-    ViewQuizListByCategory, QuizUserProgressView, QuizDetailView
+    ViewQuizListByCategory, QuizUserProgressView, QuizMarkingList,\
+    QuizMarkingDetail, QuizDetailView, QuizTake
 
 
-urlpatterns = patterns('quiz.views',
+urlpatterns = patterns('',
 
                        url(regex=r'^$',
                            view=QuizListView.as_view(),
@@ -22,12 +23,20 @@ urlpatterns = patterns('quiz.views',
                            view=QuizUserProgressView.as_view(),
                            name='quiz_progress'),
 
+                       url(regex=r'^marking/$',
+                           view=QuizMarkingList.as_view(),
+                           name='quiz_marking'),
+
+                       url(regex=r'^marking/(?P<pk>[\d.]+)/$',
+                           view=QuizMarkingDetail.as_view(),
+                           name='quiz_marking_detail'),
+
                        #  passes variable 'quiz_name' to quiz_take view
                        url(regex=r'^(?P<slug>[\w-]+)/$',
                            view=QuizDetailView.as_view(),
                            name='quiz_start_page'),
 
                        url(regex=r'^(?P<quiz_name>[\w-]+)/take/$',
-                           view='quiz_take',
+                           view=QuizTake.as_view(),
                            name='quiz_question'),
-                       )
+)
