@@ -10,9 +10,9 @@ class TF_Question(Question):
                                             " false.")
 
     def check_if_correct(self, guess):
-        if guess == "T":
+        if guess == "True":
             guess_bool = True
-        elif guess == "F":
+        elif guess == "False":
             guess_bool = False
         else:
             return False
@@ -23,15 +23,18 @@ class TF_Question(Question):
             return False
 
     def get_answers(self):
-        return [{'correct': self.check_if_correct('T'),
+        return [{'correct': self.check_if_correct("True"),
                  'content': 'True'},
-                {'correct': self.check_if_correct('F'),
+                {'correct': self.check_if_correct("False"),
                  'content': 'False'}]
 
+    def get_answers_list(self):
+        return [(True, True), (False, False)]
+
+    def answer_choice_to_string(self, guess):
+        return str(guess)
+
     class Meta:
         verbose_name = "True/False Question"
         verbose_name_plural = "True/False Questions"
         ordering = ['category']
-
-    def __unicode__(self):
-        return unicode(self.content)
